#include "HelloMultitouch.h"

using namespace GWCTutorials;

const std::string HelloMultitouch::window_title("Hello Multitouch!");

HelloMultitouch::HelloMultitouch()
: last_tick(0)
{}

// openFrameworks calls this function once before the app starts
void HelloMultitouch::setup(){
	if (!GestureWorks::Instance()->LoadGestureWorks(L"GestureWorksCore32.dll")){
		MessageBox(NULL, L"Unable to load GestureWorksCore DLL", L"Error", 0);
		return;
	}

	// Initialize with (0,0) so GestureWorks will automatically detect our screen resolution
	GestureWorks::Instance()->InitializeGestureWorks(0, 0);

	// Register our window with GestureWorks for touch input. We could also do this with an HWND.
	// Alternatively, we could use some other method of getting touch input and feed it to GestureWorks via
	// the addEvent method. 
	if (!GestureWorks::Instance()->RegisterWindowForTouchByName(std::wstring(window_title.begin(), window_title.end()))){
		MessageBox(NULL, L"Unable to register window for touch", L"Error", 0);
		return;
	}
}

// openFrameworks calls this function before every draw cycle
void HelloMultitouch::update(){
	// For consistency, we try to run GestureWorks at a constant rate - about 60Hz in this case
	clock_t this_tick(clock());
	if (this_tick - last_tick < 16) return;
	last_tick = this_tick;

	// Tell GestureWorks to process all of the data since the last frame
	GestureWorks::Instance()->ProcessFrame();

	// Get all the point events from the current frame
	std::vector<gwc::PointEvent> point_events = GestureWorks::Instance()->ConsumePointEvents();
	handlePointEvents(point_events);
}

// openFrameworks calls this function in an infinite loop until the app ends 
void HelloMultitouch::draw(){
	for (const auto& point : touch_points){
		point.second.Draw();
	}
}

void HelloMultitouch::handlePointEvents(const std::vector<gwc::PointEvent>& point_events){
	for (const auto& point : point_events){
		if (point.status == gwc::TOUCHADDED || point.status == gwc::TOUCHUPDATE){

			// By default, GestureWorks sends us coordinates in normalized screen space. To get to localized pixel
			// coordinates we need to do a conversion.
			std::pair<int, int> screen_coords(normScreenToWindowPx(point.position.x, point.position.y));

			// Update the touchpoints position
			touch_points[point.point_id] = TouchPoint(point.point_id, screen_coords.first, screen_coords.second);
		}
		else{	
			touch_points.erase(point.point_id);
		}
	}
}
